Package: MorphSim
Type: Package
Title: Simulate Discrete Character Data along Phylogenetic Trees
Version: 1.2.0
Authors@R: c(person(given = "Laura", family = "Mulvey", role = c("aut", "cre", "cph"), email = "lauramulvey479@gmail.com"),
    person(given = "Tim", family = "Brandler",role=c("aut","cph")),
    person(given = "Alessio", family = "Capobianco",role=c("aut","cph")),
    person(given = "Rachel", family = "Warnock", role=c("aut","cph")),
    person(given =  "Joelle", family = "Barido-Sottani", role=c("aut","cph")))
Description: Tools to simulate morphological traits along phylogenetic trees
 with branch lengths representing evolutionary distance or time.
 Includes functions for visualizing evolutionary processes along trees
 and within morphological character matrices.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: ape, FossilSim, stats, graphics, phangorn
Suggests: TreeSim, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
Depends: R (>= 3.5)
LazyData: true
NeedsCompilation: no
Packaged: 2026-05-06 13:45:15 UTC; lauramulvey
Author: Laura Mulvey [aut, cre, cph],
  Tim Brandler [aut, cph],
  Alessio Capobianco [aut, cph],
  Rachel Warnock [aut, cph],
  Joelle Barido-Sottani [aut, cph]
Maintainer: Laura Mulvey <lauramulvey479@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-06 14:30:02 UTC
Built: R 4.5.2; ; 2026-05-06 16:27:25 UTC; unix
