# Generated by roxygen2: do not edit by hand

S3method(as.morpho,default)
S3method(plot,morpho)
S3method(print,morpho)
S3method(summary,morpho)
export(as.morpho)
export(combine.morpho)
export(convergent_evol)
export(get.convergent)
export(get.matrix)
export(get.reconstructed)
export(get.transitions)
export(get_gamma_rates)
export(get_lognormal_rates)
export(is.morpho)
export(morpho)
export(plotMorphoGrid)
export(sim.missing.data)
export(sim.morpho)
export(stats.morpho)
export(symmetric.Q.matrix)
export(write.morpho)
export(write.recon.matrix)
export(write.recon.tree)
export(write.recon.tsv)
export(write.tsv)
import(graphics)
import(stats)
importFrom(ape,Ntip)
importFrom(ape,node.depth.edgelength)
importFrom(ape,nodepath)
importFrom(ape,write.nexus.data)
importFrom(ape,write.tree)
importFrom(phangorn,CI)
importFrom(phangorn,RF.dist)
importFrom(phangorn,RI)
importFrom(phangorn,phyDat)
