#' Example morpho dataset
#'
#' Small example dataset for testing morpho functions.
#'
#' @format A list with components:
#' \describe{
#'   \item{sequences}{List of sequences for each taxon}
#'   \item{trees}{List containing the evolutionary tree}
#' }
#' @usage data(morpho_data)
"morpho_data"
