% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reconstruct.tree.R
\name{reconstruct.tree}
\alias{reconstruct.tree}
\title{Color branches for plotting a reconstructed tree}
\usage{
reconstruct.tree(data)
}
\arguments{
\item{data}{A morpho object which contains fossil data and a time-calibrated tree.}
}
\value{
A list of length 2:
  \item{b.colours}{Vector of branch colors for plotting.}
  \item{rem}{Indices of branches with fossils.}
}
\description{
This function generates colors for branches when plotting a reconstructed tree
from a morpho object containing fossil data. Branches that are part of the
reconstructed tree or have fossils along them are colored black; all others are grey.
}
